Option Explicit On 
Option Strict On
Option Compare Binary

Imports Microsoft.VisualBasic
Imports System
Imports System.Net
Imports System.Net.Sockets
Imports System.Text

' <remarks>
'   Stellt Funktionalitt zum Einholen von 
'   WHOIS-Informationen ber eine Domain bereit.
' </remarks>
Public Class WhoisResolver
    Private m_strDomain As String
    Private m_strWhoisServer As String
    Private m_blnAutoSearch As Boolean

    ' <summary>
    '   Erstellt eine neue Instanz der Klasse.
    ' </summary>
    Public Sub New()
        Me.Domain = ""
        Me.WhoisServer = ""
        Me.AutoSearch = True
    End Sub

    ' <summary>
    '   Gibt den Namen der Domain, zu der die 
    '   Informationen ermittelt werden sollen 
    '   an oder gibt ihn zurck.
    ' </summary>
    ' <value>Name der Domain, zu der 
    '    Informationen ermittelt werden sollen.</value>
    Public Property Domain() As String
        Get
            Return m_strDomain
        End Get
        Set(ByVal Value As String)
            m_strDomain = Value
        End Set
    End Property

    ' <summary>
    '   Gibt den Namen des WHOIS-Servers, 
    '   ber den die Informationen ermittelt 
    '   werden sollen,
    '   an oder gibt ihn zurck.
    ' </summary>
    ' <value>Name des WHOIS-Servers.</value>
    '
    ' Liste mit vielen WHOIS-Servern:
    ' http://www.stanford.edu/group/networking/email/EMailBook_6.html
    '
    Public Property WhoisServer() As String
        Get
            Return m_strWhoisServer
        End Get
        Set(ByVal Value As String)
            m_strWhoisServer = Value
        End Set
    End Property

    ' <summary>
    '   Gibt an oder zurck, ob automatisch ein 
    '   passender WHOIS-Server gesucht werden soll.
    '   Hat die Eigenschaft den Wert <c>False</c>, 
    '   dann wird der in der Eigenschaft
    '   <c>WhoisServer</c> angegebene Server benutzt.
    ' </summary>
    ' <value>
    '   Boolescher Wert, der angibt, ob automatisch 
    '   ein passender WHOIS-Server gesucht werden soll.
    ' </value>
    Public Property AutoSearch() As Boolean
        Get
            Return m_blnAutoSearch
        End Get
        Set(ByVal Value As Boolean)
            m_blnAutoSearch = Value
        End Set
    End Property

    ' <summary>
    '   Stellt die Anfrage nach dem in der 
    '   Eigenschaft <c>Domain</c> angegebenen Server
    '   an einen WHOIS-Server und gibt die 
    '   ermittelten Informationen zurck.
    ' </summary>
    ' <returns>Informationen ber eine Domain, 
    '    die vom WHOIS-Server zurckgegeben wurden.</returns>
    ' <exception cref="Exception">Ein Fehler beim 
    '    stellen des Request ist aufgetreten.</exception>
    ' <exception cref="ArgumentException">Die angegebene 
    '    Domain ist ungltig.</exception>
    Public Function Resolve() As String

        ' Ermitteln des durch den Benutzer 
        ' angegebenen(WhoIs - Servers, beispielsweise)
        ' whois.nic.de:43/TCP und ablegen in einem 
        ' Endpunkt, der vom Socket verwendet wird,
        ' um zur Site zu verbinden.
        Dim strWhoisServer As String = WhoisServer
        If Me.AutoSearch Then
            Dim n As Integer = Me.Domain.LastIndexOf(".")
            If n < 0 OrElse n = Me.Domain.Length Then
                Throw New ArgumentException()
            End If
            strWhoisServer = Me.Domain.Substring(n + 1) & _
              ".whois-servers.net"
        End If

        Dim iphost As IPHostEntry
        Try
            iphost = Dns.Resolve(strWhoisServer)
        Catch ex As Exception

            ' Hier kann ein Fehler auftreten, dessen 
            ' Ursache unklar ist. Beispielsweise
            ' kann nicht auf whois.th-chemnitz.de 
            ' verbunden werden.
            Throw ex
        End Try

        ' IP-End an der erstbesten IP-Adresse whlen.
        Dim ipend As IPEndPoint = _
          New IPEndPoint(iphost.AddressList(0), 43)

        Dim strResponse As String = ""
        Dim socket As Socket = Nothing
        Try

            ' Erstellen eines Internet TCP-Sockets.
            socket = _
                New Socket( _
                    AddressFamily.InterNetwork, _
                    SocketType.Stream, _
                    ProtocolType.Tcp _
                )

            ' Verbinden zum Endpunkt.
            socket.Connect(ipend)

            ' Senden der Anfrage. Request 
            ' als Byte-Array fr das Socket 
            ' zum Senden vorbereiten.
            socket.Send(Encoding.ASCII.GetBytes( _
              Me.Domain & ControlChars.CrLf), _
              SocketFlags.None)

            ' Ermitteln der Antwort und decodieren derselben.
            Dim abytIn(1023) As Byte
            ' Allokieren von 1024 Elementen.
            Dim intReceived As Integer = socket.Receive(abytIn)
            Do While intReceived > 0
                strResponse &= _
                    Encoding.ASCII.GetString( _
                        abytIn, _
                        0, _
                        intReceived _
                    )
                intReceived = socket.Receive(abytIn)
            Loop

            ' Daten ermittelt, aufrumen.
            socket.Shutdown(SocketShutdown.Both)
            socket.Close()
        Catch ex As Exception
            Throw ex
        Finally
            If Not socket Is Nothing Then
                socket.Close()
            End If
        End Try

        ' Ermittelte Antwort zurckgeben. 
        Return strResponse
    End Function
End Class